from arcade.glui.dialog import Dialog
from arcade.glui.gamecenterrunner import GameCenterRunner
from arcade.glui.input import InputHandler
from arcade.glui.menu import Menu
from arcade.glui.opengl import fs_emu_blending, fs_emu_texturing
from OpenGL import GL as gl

from arcade.glui.render import Render
from arcade.glui.state import State
from arcade.glui.texture import Texture
from arcade.glui.topmenu import GameCenterItem
from arcade.glui.window import (
    back_to_menu_from_game,
    render_fade,
    set_current_menu,
    set_ingame_status,
)

STATE_STARTING = 0
STATE_PREPARING = 1
STATE_WAITRUN = 2
STATE_RUNNING = 3
STATE_STOPPING = 4
STATE_ABORTING = 5
STATE_ABORTED = 6

FADE_TIME = 0.5


class LaunchMenu(Menu):
    def __init__(self, item, controller):
        Menu.__init__(self)
        # self.top_menu_transition = 0.0
        self.items.append(item)
        if self.use_game_center_item():
            self.top.left.append(GameCenterItem())
        # self.top.left.append(HomeItem())
        # self.top.left.append(MenuItem(item.title))
        self.top.set_selected_index(
            len(self.top.left) + len(self.top.right) - 1
        )
        self.state = STATE_STARTING
        self.gc_runner = None
        self.controller = controller
        self.throbber = Throbber()
        self.wait_run_time = 0

    # noinspection PyMethodMayBeStatic
    def on_status(self, status):
        print("received status", status)

    def update_state(self):
        if self.state == STATE_STARTING:
            self.gc_runner = GameCenterRunner(controller=self.controller)
            # prepare will unpack the game and prepare game files
            self.gc_runner.prepare()
            self.state = STATE_PREPARING
            self.throbber.throbber_start_time = 0

        elif self.state == STATE_PREPARING:
            if self.gc_runner.error:
                pass
            elif self.gc_runner.done:
                self.wait_run_time = State.get().time
                FadeDialog(self.wait_run_time).show()
                self.state = STATE_WAITRUN

        elif self.state == STATE_WAITRUN:
            if State.get().time - self.wait_run_time > FADE_TIME:
                set_ingame_status()
                self.gc_runner.run()
                self.state = STATE_RUNNING

        elif self.state == STATE_RUNNING:
            InputHandler.set_inhibited(True)
            if self.gc_runner.error:
                pass
            elif self.gc_runner.done:
                self.state = STATE_STOPPING

        elif self.state == STATE_STOPPING:
            InputHandler.set_inhibited(False)
            State.get().history.pop()
            State.get().history.pop()
            State.get().history.pop()
            set_current_menu(State.get().history[-1])
            back_to_menu_from_game()
            Dialog.get_current().close()

            from arcade.glui.window import main_window

            main_window.restore_window_if_necessary()

        elif self.state == STATE_ABORTING:
            pass

    def go_back(self):
        print("LaunchMenu.go_back")
        if self.state in [STATE_STARTING, STATE_PREPARING]:
            if self.gc_runner:
                self.gc_runner.abort()
                self.state = STATE_ABORTING
        if self.state in [STATE_ABORTED]:
            State.get().history.pop()
            set_current_menu(State.get().history[-1])

    def activate(self):
        print("LaunchMenu.activate")

    def update(self):
        self.update_state()

    def render(self):
        Render.get().hd_perspective()
        fs_emu_texturing(True)
        fs_emu_blending(False)
        Texture.sidebar_background.render(
            0, 0, 1920, Texture.sidebar_background.h
        )

        if self.state in [STATE_PREPARING, STATE_WAITRUN]:
            self.throbber.render()
            # elif self.state in [STATE_RUNNING, STATE_STOPPING]:
            #     pass
            # gl.glDisable(gl.GL_DEPTH_TEST)
            # render_fade(0.0, 0.0, 0.0, 1.0)
            # gl.glEnable(gl.GL_DEPTH_TEST)


class FadeDialog(Dialog):
    def __init__(self, wait_run_time):
        super().__init__()
        self.wait_run_time = wait_run_time

    def render(self):
        gl.glDisable(gl.GL_DEPTH_TEST)
        alpha = (State.get().time - self.wait_run_time) / FADE_TIME
        render_fade(0.0, 0.0, 0.0, alpha)
        gl.glEnable(gl.GL_DEPTH_TEST)


class Throbber:
    def __init__(self):
        self.throbber_colors = [1.0, 0.8, 0.6, 0.2, 0.2, 0.2, 0.2, 0.2]
        self.throbber_start_time = 0
        self.throbber_progress = 0
        self.throbber_opacity = 1.0
        # self.start_time = 0

    def render(self):
        Render.get().hd_perspective()
        if self.throbber_start_time == 0:
            self.throbber_start_time = State.get().time
        dt = State.get().time - self.throbber_start_time
        # run animation with 15 fps
        self.throbber_progress = int(dt * 15)

        # bg_fade = (State.get().time - State.get().dialog_time) / 0.5
        # if bg_fade > 1.0:
        #     bg_fade = 1.0
        # elif bg_fade < 0.0:
        #     bg_fade = 0.0
        fs_emu_texturing(False)
        fs_emu_blending(True)
        gl.glDisable(gl.GL_DEPTH_TEST)
        gl.glDepthMask(False)

        # gl.glBegin(gl.GL_QUADS)
        # gl.glColor4f(0.0, 0.0, 0.0, bg_fade)
        # gl.glVertex2f(0, 0)
        # gl.glVertex2f(1920, 0)
        # gl.glVertex2f(1920, 1020)
        # gl.glVertex2f(0, 1020)
        # gl.glEnd()

        # gl.glBegin(gl.GL_QUADS)
        # gl.glColor4f(0.0, 0.0, 0.0, bg_fade * 0.5)
        # gl.glVertex2f(0, 1020)
        # gl.glVertex2f(1920, 1020)
        # gl.glVertex2f(1920, 1080)
        # gl.glVertex2f(0, 1080)
        # gl.glEnd()

        # y = 0.0
        # tw, th = Render.get().text("LAUNCHING GAME", self.title_font,
        #         0.0, y, w=32 / 9, h=self.height,
        #         color=(1.0, 1.0, 1.0, 1.0), halign=0.0)
        #
        fs_emu_blending(True)
        # if bg_fade > 0.5:
        #     self.throbber_opacity = (bg_fade - 0.5) / 0.5
        #     #self.throbber_opacity = bg_fade
        #     self.render_throbber()
        # if bg_fade == 1.0:
        # if State.get().time - State.get().dialog_time > 1.0:
        #     # gradually show over 1/4 second
        #     self.throbber_opacity = (
        #         (State.get().time - State.get().dialog_time - 1.0) * 4)
        #     if self.throbber_opacity > 1.0:
        #         self.throbber_opacity = 1.0
        #     self.render_throbber()
        if State.get().time - self.throbber_start_time > 0.25:
            # gradually show over 1/3 second
            self.throbber_opacity = (
                State.get().time - self.throbber_start_time - 0.25
            ) * 4
            if self.throbber_opacity > 1.0:
                self.throbber_opacity = 1.0
            self.render_throbber()

        gl.glEnable(gl.GL_DEPTH_TEST)
        gl.glDepthMask(True)
        # fs_emu_texturing(1)
        # fs_emu_blending(0)
        # print("Setting dirty..")
        Render.get().dirty = True

    def render_throbber(self):
        cell_width = 32
        cell_spacing = 16
        throbber_width = 3 * cell_width + 2 * cell_spacing
        # throbber_height = throbber_width
        left = (1920 - throbber_width) / 2
        bottom = (1080 - throbber_width) / 2

        self.render_cell(0, left, bottom)
        self.render_cell(1, left + 1 * cell_width + 1 * cell_spacing, bottom)
        self.render_cell(2, left + 2 * cell_width + 2 * cell_spacing, bottom)
        self.render_cell(
            3,
            left + 2 * cell_width + 2 * cell_spacing,
            bottom + 1 * cell_width + 1 * cell_spacing,
        )
        self.render_cell(
            4,
            left + 2 * cell_width + 2 * cell_spacing,
            bottom + 2 * cell_width + 2 * cell_spacing,
        )
        self.render_cell(
            5,
            left + 1 * cell_width + 1 * cell_spacing,
            bottom + 2 * cell_width + 2 * cell_spacing,
        )
        self.render_cell(6, left, bottom + 2 * cell_width + 2 * cell_spacing)
        self.render_cell(7, left, bottom + 1 * cell_width + 1 * cell_spacing)

    def render_cell(self, index, x, y):
        color = self.throbber_colors[(self.throbber_progress + index) % 8]
        color *= self.throbber_opacity
        gl.glColor4f(color, color, color, self.throbber_opacity)
        gl.glBegin(gl.GL_QUADS)
        gl.glVertex2f(x, y)
        gl.glVertex2f(x + 32, y)
        gl.glVertex2f(x + 32, y + 32)
        gl.glVertex2f(x, y + 32)
        gl.glEnd()



import platform
import sys
import webbrowser


def default_url_open_function(url: str):
    webbrowser.open(url)


_url_open_function = default_url_open_function


def open_url_in_browser(url: str) -> None:
    if _url_open_function:
        _url_open_function(url)


def set_open_url_in_browser_function(func):
    global _url_open_function
    _url_open_function = func


def is_running_gnome_3():
    import fstd.desktop.gnome3

    return fstd.desktop.gnome3.running_in_gnome_3()


def is_running_windows_10():
    if sys.platform != "win32":
        return False
    return platform.uname()[2] == "10"



import warnings

from fsui.qt import QParent, QPushButton, QSignal
from fsui.qt.widget import Widget


class Button(Widget):
    activated = QSignal()

    def __init__(self, parent, label=""):
        super().__init__(parent)
        label = "  " + label + "  "
        self.set_widget(QPushButton(label, QParent(parent)))
        self._widget.clicked.connect(self.__clicked)

    def __clicked(self):
        self.on_activated()

    def on_activated(self):
        self.activated.emit()
        # FIXME: Remove
        self.on_activate()

    def on_activate(self):
        pass

    @property
    def clicked(self):
        warnings.warn("use activated instead", DeprecationWarning)
        return self.activated



import sys


def app_main():
    print("FS Game Center")

    from fsui import Application

    application = Application("fs-game-center")

    import threading

    from fsgs.http.server import http_server_main

    def http_server_thread():
        return http_server_main()

    threading.Thread(target=http_server_thread).start()

    if sys.platform.startswith("linux"):
        # Must load OpenGL to work around crash with Qt Quick on
        # Nvidia / Linux (due to libgl.so from mesa being loaded).
        import ctypes

        ctypes.CDLL("libGL.so.1", ctypes.RTLD_GLOBAL)

    from fsgs.ui.qwindow import GameCenterView

    window = GameCenterView()
    from fsui.qt import Qt

    # window.setFlags(Qt.WindowType.FramelessWindowHint)
    window.setFlags(
        Qt.WindowType.Window
        | Qt.WindowType.FramelessWindowHint
        | Qt.WindowType.WindowMinimizeButtonHint
        | Qt.WindowType.WindowSystemMenuHint
    )
    # window.show()
    window.showMaximized()
    # window.showFullScreen()
    # window.setWindowState(Qt.WindowState.WindowFullScreen)
    # window.setVisible(True)

    # from fs_uae_launcher.FSUAELauncher import FSUAELauncher
    # application = FSUAELauncher()

    # if application.start():
    application.run()
    # application.save_settings()

    from fsbc.signal import Signal

    Signal("quit").notify()



import fsui
from fsgs.option import Option
from launcher.devicemanager import DeviceManager
from launcher.i18n import gettext
from launcher.launcher_settings import LauncherSettings
from launcher.settings.settings_page import SettingsPage
from workspace.apps.joystick_config_app import JoystickConfigWindow


class JoystickSettingsPage(SettingsPage):
    def __init__(self, parent):
        super().__init__(parent)
        icon = fsui.Icon("joystick-settings", "pkg:workspace")
        # gettext("Joystick Settings")
        title = gettext("Controllers")
        subtitle = gettext(
            "Configure joysticks and set preferred joystick devices"
        )
        self.add_header(icon, title, subtitle)

        label = fsui.Label(
            self, gettext("Double-click a controller to configure it:")
        )
        self.layout.add(label, margin_bottom=10)

        self.list_view = fsui.ListView(self)
        self.list_view.set_min_height(140)
        self.list_view.item_activated.connect(self.on_joystick_activated)
        image = fsui.Image("workspace:res/16x16/gamepad.png")
        for device_name in DeviceManager.get_joystick_names():
            if DeviceManager.is_joystick(device_name):
                self.list_view.add_item(device_name, icon=image)
        self.layout.add(self.list_view, fill=True, expand=True)

        self.layout.add_spacer(20)
        self.pref_group = PreferredJoysticksGroup(self)
        self.layout.add(self.pref_group, fill=True)

        # For reset to defaults function
        self.options_on_page.add(Option.PRIMARY_JOYSTICK)
        self.options_on_page.add(Option.SECONDARY_JOYSTICK)

    def on_joystick_activated(self, index):
        device_name = self.list_view.get_item(index)
        print(self.get_window())
        # shell_open("Workspace:Tools/JoystickConfig", [device_name],
        #            parent=self.get_window())
        JoystickConfigWindow(self.window, device_name).show()


joystick_mode_values = ["nothing", "mouse", "joystick"]
joystick_values = ["none", "mouse", "keyboard"]


class PreferredJoysticksGroup(fsui.Group):
    def __init__(self, parent):
        fsui.Group.__init__(self, parent)
        self.layout = fsui.HorizontalLayout()
        # self.layout.padding_left = 10
        # self.layout.padding_right = 10

        # image = fsui.Image("launcher:res/joystick.png")
        # self.image_view = fsui.ImageView(self, image)
        # self.layout.add(self.image_view, valign=0.0)

        # self.layout.add_spacer(20)

        self.layout2 = fsui.VerticalLayout()
        self.layout.add(self.layout2, fill=True, expand=True)

        heading = gettext("Preferred Controllers")
        label = fsui.HeadingLabel(self, heading)
        self.layout2.add(label)

        self.layout2.add_spacer(20)
        label = fsui.Label(self, gettext("Preferred controller (if present):"))
        self.layout2.add(label)

        self.layout2.add_spacer(6)
        selector = PreferredJoystickSelector(self, 0)
        self.layout2.add(selector, fill=True)

        self.layout2.add_spacer(20)
        label = fsui.Label(
            self, gettext("Preferred device for secondary controller:")
        )
        self.layout2.add(label)

        self.layout2.add_spacer(6)
        selector = PreferredJoystickSelector(self, 1)
        self.layout2.add(selector, fill=True)


class PreferredJoystickSelector(fsui.Group):
    def __init__(self, parent, index):
        self.index = index
        if index:
            self.key = Option.SECONDARY_JOYSTICK
        else:
            self.key = Option.PRIMARY_JOYSTICK

        fsui.Group.__init__(self, parent)
        self.layout = fsui.HorizontalLayout()

        devices = ["", get_keyboard_title()]
        # for i, name in enumerate(DeviceManager.get_joystick_names()):
        #     devices.append(name)
        for device_name in DeviceManager.get_joystick_names():
            if DeviceManager.is_joystick(device_name):
                devices.append(device_name)

        self.device_choice = fsui.ComboBox(self, devices)

        self.layout.add(self.device_choice, expand=True)

        # Config.add_listener(self)

        self.initialize_from_settings()
        self.set_settings_handlers()

    def initialize_from_settings(self):
        self.on_setting(self.key, LauncherSettings.get(self.key))

    def set_settings_handlers(self):
        self.device_choice.on_changed = self.on_device_changed
        LauncherSettings.add_listener(self)

    def on_destroy(self):
        LauncherSettings.remove_listener(self)

    def on_device_changed(self):
        value = self.device_choice.get_text()
        print("on_device_change", value)
        if value == get_keyboard_title():
            value = "keyboard"
        LauncherSettings.set(self.key, value)

    def on_setting(self, key, value):
        if key == self.key:
            if value == "keyboard":
                value = get_keyboard_title()
            self.device_choice.set_text(value)


def get_keyboard_title():
    return gettext("Cursor Keys and Right Ctrl/Alt")



from fsui import Image

from ...i18n import gettext
from ...launcher_config import LauncherConfig
from .StatusElement import StatusElement


class ProtectionElement(StatusElement):
    def __init__(self, parent):
        StatusElement.__init__(self, parent)
        # self.set_min_width(140)
        # self.layout = HorizontalLayout()
        self.protection_icon = Image("launcher:res/16x16/lock.png")
        # self.unknown_icon = self.icon.grey_scale()
        self.disabled_icon = Image("launcher:res/16x16/lock_open_green.png")
        # self.disabled_icon = self.disabled_icon.grey_scale()
        self.icon = self.protection_icon

        self.na_text = " "
        self.protection = ""
        self.text = self.na_text
        self.active = False

        LauncherConfig.add_listener(self)
        self.on_config("protection", LauncherConfig.get("protection"))

    def on_destroy(self):
        LauncherConfig.remove_listener(self)

    def on_config(self, key, value):
        if key == "protection":
            print(" -- protection --", value)
            if value != self.protection:
                self.protection = value
                if not value:
                    self.icon = self.protection_icon
                    self.active = False
                    self.text = self.na_text
                elif value.lower() == "none":
                    # self.icon = self.disabled_icon
                    # self.active = True
                    self.icon = self.protection_icon
                    self.active = False
                    self.text = gettext("No Protection")
                elif value.lower() in ["dongle"]:
                    # self.icon = self.disabled_icon
                    # self.active = True
                    self.icon = self.protection_icon
                    self.active = False
                    self.text = gettext("Dongle")
                else:
                    self.icon = self.protection_icon
                    self.active = True
                    self.text = value
                self.refresh()



from __future__ import print_function

import zipfile
import StringIO

# optional lhafile
try:
  import lhafile
except ImportError:
  lhafile = None


class ArchiveScanner:
  """Scan archives and visit all files"""

  exts = [] # valid file extensions

  def _create_archive_obj(self, fobj, scanner):
    pass

  def _create_entry_scan_file(self, arc, info, sf):
    pass

  def can_handle(self, scan_file):
    base_name = scan_file.get_basename().lower()
    for ext in self.exts:
      if base_name.endswith(ext):
        return True
    return False

  def handle(self, scan_file, scanner):
    """scan a given archive file"""
    # ensure a seekable fobj
    if not scan_file.is_seekable():
      sf = scanner.promote_scan_file(scan_file, seekable=True)
    else:
      sf = scan_file
    # create archive obj
    arc = self._create_archive_obj(sf, scanner)
    if arc is None:
      return True # simply ignore
    # get infos
    infos = arc.infolist()
    for info in infos:
      if info.file_size > 0:
        sf = self._create_entry_scan_file(arc, info, scan_file)
        ok = scanner.scan_obj(sf)
        sf.close()
        if not ok:
          return False
    return True


class ZipScanner(ArchiveScanner):
  """Scan .zip Archives"""

  exts = [".zip"]

  def _create_archive_obj(self, sf, scanner):
    try:
      fobj = sf.get_fobj()
      return zipfile.ZipFile(fobj, "r")
    except Exception as e:
      scanner.warn(sf, "error reading archive: %s" % e)

  def _create_entry_scan_file(self, arc, info, scan_file):
    name = info.filename
    fobj = arc.open(info)
    size = info.file_size
    # its a non-seekable file
    return scan_file.create_sub_path(name, fobj, size, False, False)


class LhaScanner(ArchiveScanner):
  """Scan .lha/.lzh Archives"""

  exts = [".lha", ".lzh"]

  def _create_archive_obj(self, sf, scanner):
    if lhafile:
      try:
        fobj = sf.get_fobj()
        return lhafile.LhaFile(fobj, "r")
      except Exception as e:
        scanner.warn(sf, "error reading archive: %s" % e)
    else:
      scanner.warn(sf, "can't handle archive. missing 'lhafile' module.")

  def _create_entry_scan_file(self, arc, info, scan_file):
    data = arc.read(info.filename)
    fobj = StringIO.StringIO(data)
    size = info.file_size
    name = info.filename
    return scan_file.create_sub_path(name, fobj, size, True, False)


# mini test
if __name__ == '__main__':
  import sys
  from FileScanner import FileScanner

  ifs = ['*.txt']
  def handler(scan_file):
    print(scan_file)
    return True
  def skip_handler(scan_file):
    print("SKIP:", scan_file)
    return True
  def error_handler(scan_file, error):
    print("FAILED:", scan_file, error)
    raise error
  scanners = [LhaScanner(), ZipScanner()]
  fs = FileScanner(handler, ignore_filters=ifs, error_handler=error_handler,
                   scanners=scanners, skip_handler=skip_handler)
  for a in sys.argv[1:]:
    fs.scan(a)



import hashlib
import os
import traceback

from fsgs.amiga.adf import ADFFile

from .workbenchdata import (
    wb_204_files,
    wb_204_floppies,
    wb_204_startup_sequence,
    wb_300_files,
    wb_300_floppies,
    wb_300_startup_sequence,
)


class WorkbenchExtractor(object):
    def __init__(self, fsgs):
        self.fsgs = fsgs
        self.cached_adf = None
        self.cached_adf_sha1 = None

    def install_version(
        self, version, dest_dir, file_names=None, install_startup_sequence=True
    ):
        if version == "2.04":
            startup_sequence = wb_204_startup_sequence
            files = wb_204_files
            floppies = wb_204_floppies
        elif version == "3.0":
            startup_sequence = wb_300_startup_sequence
            files = wb_300_files
            floppies = wb_300_floppies
        else:
            raise Exception("Unsupported WB version")

        for name, sha1 in files.items():
            if file_names is not None and name not in file_names:
                continue
            name = name.rstrip("/")
            dest_path = os.path.join(dest_dir, name)
            if not sha1:
                if not os.path.exists(dest_path):
                    os.makedirs(dest_path)
                continue
            self.copy_workbench_file(name, sha1, dest_path, floppies)

        if install_startup_sequence:
            with open(
                os.path.join(dest_dir, "S", "Startup-Sequence"), "wb"
            ) as f:
                f.write(
                    startup_sequence.replace("\r\n", "\n").encode("ISO-8859-1")
                )

    def copy_workbench_file(self, name, sha1, dest_path, floppies):
        print("copying workbench file", name)
        if not os.path.exists(os.path.dirname(dest_path)):
            os.makedirs(os.path.dirname(dest_path))
        # for floppy_sha1, name in workbench_file_map[sha1]:

        if self.cached_adf is not None:
            # using a cached/in-memory ADF object is very efficient here
            # compared to parsing the ADF from scratch for every file
            if self.cached_adf_sha1 in floppies:
                result = self.extract_workbench_file(
                    self.cached_adf, name, dest_path
                )
                assert result == sha1
                return

        for floppy_sha1 in floppies:
            path = self.fsgs.file.find_by_sha1(floppy_sha1)
            if path:
                try:
                    input_stream = self.fsgs.file.open(path)
                except Exception:
                    traceback.print_exc()
                else:
                    wb_data = input_stream.read()
                    adf = ADFFile(wb_data)
                    self.cached_adf = adf
                    self.cached_adf_sha1 = floppy_sha1
                    result = self.extract_workbench_file(adf, name, dest_path)
                    assert result == sha1
                    return
        else:
            raise Exception("Could not find workbench file {0}".format(sha1))

    def extract_workbench_file(self, adf, name, dest):
        input_stream = adf.open(name, "r")
        data = input_stream.read()
        with open(dest, "wb") as f:
            f.write(data)
        return hashlib.sha1(data).hexdigest()



import os
import sys


def unicode_path(path):
    if isinstance(path, str):
        return path
    return path.decode(sys.getfilesystemencoding())


def str_path(path):
    return path


def normalize_path(path):
    path = os.path.normcase(os.path.normpath(path))
    return unicode_path(path)


def is_same_file(path_a, path_b):
    """Checks whether path_a and path_b refer to the same file. It does this
    by normalizing the paths (and case if applicable) and eliminating any
    symbolic links along the way."""
    path_a = normalize_path(os.path.realpath(path_a))
    path_b = normalize_path(os.path.realpath(path_b))
    return path_a == path_b



# noinspection PyUnresolvedReferences
# noinspection PyUnresolvedReferences
from OpenGL.arrays.vbo import VBO
from OpenGL.GL import *

# noinspection PyUnresolvedReferences
from OpenGL.GL.ARB.fragment_shader import *

# noinspection PyUnresolvedReferences
from OpenGL.GL.ARB.framebuffer_object import *

# noinspection PyUnresolvedReferences
from OpenGL.GL.ARB.multitexture import *

# noinspection PyUnresolvedReferences
from OpenGL.GL.ARB.shader_objects import *

# noinspection PyUnresolvedReferences
from OpenGL.GL.ARB.vertex_shader import *

# noinspection PyUnresolvedReferences
from OpenGL.GLU import *

# noinspection PyUnresolvedReferences
from OpenGL.raw.GL.EXT.framebuffer_object import *

# from OpenGLContext.arrays import *


def filter_global(x):
    if x.lower().startswith("gl"):
        return True
    return False


def pyopengl_globals():
    return globals()
